@@ -9,68 +9,96 @@ options(digits=4, prompt="R> ")
 
 
 ###################################################
-### code chunk number 2: qle_with_R.Rnw:1152-1156
+### code chunk number 2: qle_with_R.Rnw:1168-1170
 ###################################################
 library(qle)
-library(graphics)
-RNGkind("L'Ecuyer-CMRG")
-set.seed(1356)
+data(mm1q)
 
 
 ###################################################
-### code chunk number 3: qle_with_R.Rnw:1162-1166
+### code chunk number 3: qle_with_R.Rnw:1173-1177 (eval = FALSE)
 ###################################################
-cond <- list("n"=25)
+## options(mc.cores=8L)
+## options(qle.multicore="mclapply")
+## RNGkind("L'Ecuyer-CMRG")
+## set.seed(1326)
+
+
+###################################################
+### code chunk number 4: qle_with_R.Rnw:1184-1188
+###################################################
+cond <- list("n"=100)
 simfn <- function(tet,cond){
 	mean(rgeom(cond$n,prob=1-tet[1]))
 }
 
 
 ###################################################
-### code chunk number 4: qle_with_R.Rnw:1170-1172
+### code chunk number 5: qle_with_R.Rnw:1192-1194
 ###################################################
 lb <- c("rho"=0.05)
 ub <- c("rho"=0.95)
 
 
 ###################################################
-### code chunk number 5: qle_with_R.Rnw:1178-1183
+### code chunk number 6: qle_with_R.Rnw:1200-1204 (eval = FALSE)
 ###################################################
-nsim <- 10
-X <- multiDimLHS(N=10,lb=lb,ub=ub,
-      method="maximinLHS",type="matrix")
-
-sim <- simQLdata(sim=simfn,cond=cond,nsim=nsim,X=X)
+## nsim <- 10
+## X <- multiDimLHS(N=9,lb=lb,ub=ub,
+##       method="maximinLHS",type="matrix")
+## sim <- simQLdata(sim=simfn,cond=cond,nsim=nsim,X=X)
 
 
 ###################################################
-### code chunk number 6: qle_with_R.Rnw:1189-1191
+### code chunk number 7: qle_with_R.Rnw:1206-1209
 ###################################################
-qsd <- getQLmodel(sim, lb, ub, obs=c("N"=1),
-		var.type="wlogMean",verbose=TRUE)
+sim <- mm1q$sim
+X <- attr(sim,"X")
+nsim <- attr(sim,"nsim")
 
 
 ###################################################
-### code chunk number 7: qle_with_R.Rnw:1195-1197
+### code chunk number 8: qle_with_R.Rnw:1215-1217 (eval = FALSE)
+###################################################
+## qsd <- getQLmodel(sim, lb, ub, obs=c("N"=1),
+## 		var.type="wlogMean",verbose=TRUE)
+
+
+###################################################
+### code chunk number 9: qle_with_R.Rnw:1221-1222 (eval = FALSE)
+###################################################
+## S0 <- qscoring(qsd,x0=c("rho"=0.8))
+
+
+###################################################
+### code chunk number 10: qle_with_R.Rnw:1224-1227
+###################################################
+OPT <- mm1q$OPT
+qsd <- mm1q$qsd
+S0 <- mm1q$S0
+
+
+###################################################
+### code chunk number 11: qle_with_R.Rnw:1229-1230
 ###################################################
-S0 <- qscoring(qsd,x0=c("rho"=0.8))
 print(S0)
 
 
 ###################################################
-### code chunk number 8: qle_with_R.Rnw:1212-1217
+### code chunk number 12: qle_with_R.Rnw:1244-1249 (eval = FALSE)
 ###################################################
-OPT <- qle(qsd,simfn,cond=cond,	     	
-  global.opts = list("maxeval"=5, "NmaxLam"=5),
-  local.opts = list("nextSample"="score","weights"=0.5,"ftol_abs"=1e-4,
-                    "lam_max"=1e-5,"test"=TRUE),
-  method = c("qscoring","bobyqa","direct"), iseed=1356) 
+## OPT <- qle(qsd,simfn,cond=cond,	     	
+##   global.opts = list("maxeval"=5, "NmaxLam"=5),
+##   local.opts = list("nextSample"="score","weights"=0.5,
+##                     "ftol_abs"=1e-4, "lam_max"=1e-5),
+##   method = c("qscoring","bobyqa","direct"), iseed=1326) 
 
 
 ###################################################
-### code chunk number 9: qle_with_R.Rnw:1222-1240
+### code chunk number 13: qle_with_R.Rnw:1254-1273
 ###################################################
-## statistics
+library(graphics)
+# statistics
 op <- par(xaxs='i', yaxs='i')
 rho <- as.matrix(seq(0.1,0.9,by=0.001))
 y <- as.numeric(unlist(simQLdata(sim=simfn,cond=cond,nsim=nsim,X=rho,mode="mean")))
@@ -91,7 +119,7 @@ par(op)
 
 
 ###################################################
-### code chunk number 10: qle_with_R.Rnw:1242-1264
+### code chunk number 14: qle_with_R.Rnw:1275-1297
 ###################################################
 op <- par(xaxs='i', yaxs='i')
 p <- seq(lb,ub,by=0.0001)
@@ -106,7 +134,7 @@ abline(h=0,col="gray")
 points(X,QD,pch=3,cex=1)
 lines(p,score, type='l',col="blue",lwd=1.5) 
 lines(p,y,col="black",lwd=0.8)
-legend("topleft", c("quasi-deviance","quasi-score","sample points", "approximate root","additional samples"),
+legend("topright", c("quasi-deviance","quasi-score","sample points", "approximate root","additional samples"),
 		lty=c(1,1),lwd=c(1.5,1.5,NA,NA,NA),pch=c(NA,NA,3,5,8),
 		col=c("black","blue","black","magenta","green"),pt.cex=1,cex=1)
 points(S0$par,S0$val,col="magenta",pch=5,cex=1)
@@ -118,13 +146,13 @@ par(op)
 
 
 ###################################################
-### code chunk number 11: qle_with_R.Rnw:1272-1273
+### code chunk number 15: qle_with_R.Rnw:1305-1306
 ###################################################
 OPT
 
 
 ###################################################
-### code chunk number 12: qle_with_R.Rnw:1278-1297
+### code chunk number 16: qle_with_R.Rnw:1311-1330
 ###################################################
 op <-par(xaxs='i', yaxs='i')
 qd <- quasiDeviance(as.matrix(p),OPT$qsd)
@@ -136,7 +164,7 @@ plot(NULL, type="n", xlab=expression(rho),
 abline(h=0,col="gray")
 lines(p,score, type='l',col="blue",lwd=1.5) 
 lines(p,y,col="black",lwd=0.8)
-legend("topleft", c("quasi-deviance","quasi-score","sample points", "QL estimate"),
+legend("topright", c("quasi-deviance","quasi-score","sample points", "QL estimate"),
 		lty=c(1,1),lwd=c(1,1,NA,NA,NA),pch=c(NA,NA,3,5,8),
 		col=c("black","blue","black","magenta","green"),pt.cex=1,cex=1)
 
@@ -148,13 +176,13 @@ par(op)
 
 
 ###################################################
-### code chunk number 13: qle_with_R.Rnw:1307-1308
+### code chunk number 17: qle_with_R.Rnw:1340-1341
 ###################################################
 checkMultRoot(OPT,verbose = TRUE)
 
 
 ###################################################
-### code chunk number 14: qle_with_R.Rnw:1318-1321
+### code chunk number 18: qle_with_R.Rnw:1351-1354
 ###################################################
 X <- as.matrix(OPT$qsd$qldata[,1])
 Tstat <- OPT$qsd$qldata[grep("mean.",names(qsd$qldata))]   
@@ -162,14 +190,21 @@ predictKM(OPT$qsd$covT,c("rho"=0.5),X,Tstat)
 
 
 ###################################################
-### code chunk number 15: qle_with_R.Rnw:1356-1358
+### code chunk number 19: qle_with_R.Rnw:1393-1395 (eval = FALSE)
 ###################################################
-tet0 <- c("rho"=0.5)
-obs0 <- simQLdata(sim=simfn,cond=cond,nsim=100,X=tet0)
+## tet0 <- c("rho"=0.5)
+## obs0 <- simQLdata(sim=simfn,cond=cond,nsim=100,X=tet0)
 
 
 ###################################################
-### code chunk number 16: qle_with_R.Rnw:1363-1370
+### code chunk number 20: qle_with_R.Rnw:1397-1399
+###################################################
+tet0 <- mm1q$tet0
+obs0 <- mm1q$obs0
+
+
+###################################################
+### code chunk number 21: qle_with_R.Rnw:1404-1411
 ###################################################
 mle <- do.call(rbind,
 		lapply(obs0[[1]],function(y,n){
@@ -181,106 +216,141 @@ mle.var <- c(sum(x^2)/length(x),mean(mle[,2]))
 
 
 ###################################################
-### code chunk number 17: qle_with_R.Rnw:1374-1392
+### code chunk number 22: qle_with_R.Rnw:1415-1427 (eval = FALSE)
 ###################################################
-cl <- makeCluster(2)
-clusterSetRNGStream(cl)
-clusterExport(cl,list("qle"))
-OPTS <- parLapplyLB(cl,obs0[[1]],
-          function(obs,...) qle(...,obs=obs),
-         qsd=qsd,
-         sim=simfn, 
-         cond=cond,
-         global.opts = list("maxeval"=5,"NmaxLam"=5),
-         local.opts = list("nextSample"="score","weights"=0.5,
-				"ftol_abs"=1e-4,"lam_max"=1e-5,
-				"useWeights"=TRUE),
-         method = c("qscoring","bobyqa","direct"))
-stopCluster(cl)
-QLE <- do.call(rbind,lapply(OPTS,
- function(x) c("qle"=x$par,"qle.var"=1/as.numeric(x$final$I))))	
+## OPTS <- parLapply(cl,obs0[[1]],
+##           function(obs,...) {
+##            qle(...,obs=obs)
+##           },
+##          qsd=qsd,
+##          sim=simfn, 
+##          cond=cond,
+##          global.opts=list("maxeval"=10,"NmaxLam"=10),
+##          local.opts=list("nextSample"="score","weights"=0.5,
+##                          "ftol_abs"=1e-4,"lam_max"=1e-5,
+##                          "useWeights"=TRUE),
+##          method=c("qscoring","bobyqa","direct"))
+
+
+###################################################
+### code chunk number 23: qle_with_R.Rnw:1429-1430
+###################################################
+OPTS <- mm1q$OPTS
+
+
+###################################################
+### code chunk number 24: qle_with_R.Rnw:1432-1441
+###################################################
+# get results
+QLE <- do.call(rbind,
+        lapply(OPTS,
+          function(x) {
+           c("qle"=x$par,"qle.var"=1/as.numeric(x$final$I))
+		  }))
 y <- QLE[,1]-tet0
+# MSE and average estimated variance of the parameters
 qle.var <- c(sum(y^2)/length(y),mean(QLE[,2]))
 
 
 ###################################################
-### code chunk number 18: qle_with_R.Rnw:1424-1426
+### code chunk number 25: qle_with_R.Rnw:1443-1444
 ###################################################
-Stest <- qleTest(OPT,sim=simfn,cond=cond,obs=obs0)
-Stest
+Stest0 <- mm1q$Stest0
 
 
 ###################################################
-### code chunk number 19: qle_with_R.Rnw:1436-1438
+### code chunk number 26: qle_with_R.Rnw:1473-1474 (eval = FALSE)
 ###################################################
-rbind("Var"=c("test"=attr(Stest,"aiqm"),"study"=qle.var[2]),
-	  "MSE"=c(as.vector(Stest$param["RMSE"]^2),qle.var[1]))
+## Stest0 <- qleTest(OPT,sim=simfn,cond=cond,obs=obs0,cl=cl)
 
 
 ###################################################
-### code chunk number 20: qle_with_R.Rnw:1454-1462
+### code chunk number 27: qle_with_R.Rnw:1476-1477
 ###################################################
-set.seed(123)
-# setting the number of cores>1
-# uses parallel simulations and computations
-options(mc.cores=2)
-simfunc <- function(pars) {	
-	x <- rnorm(10,mean=pars["mu"],sd=pars["sigma"])    
-	c("T1"=median(x),"T2"=mad(x))	
-}
+print(Stest0)
 
 
 ###################################################
-### code chunk number 21: qle_with_R.Rnw:1467-1469
+### code chunk number 28: qle_with_R.Rnw:1506-1510
 ###################################################
-lb <- c("mu"=0.5,"sigma"=0.1)
-ub <- c("mu"=8.0,"sigma"=5.0)
+data(normal)
+OPT <- qsd$OPT
+QS <- qsd$QS
+simfunc <- qsd$simfn
 
 
 ###################################################
-### code chunk number 22: qle_with_R.Rnw:1475-1480
+### code chunk number 29: qle_with_R.Rnw:1512-1520 (eval = FALSE)
 ###################################################
-sim <- simQLdata(sim=simfunc,
-           nsim=10,N=8,lb=lb,ub=ub,
-              method="maximinLHS")
-# reset number of simulations	  
-attr(sim,"nsim") <- 100
+## # use a local cluster
+## cl <- makeCluster(8L)
+## clusterSetRNGStream(cl,1234)
+## # simulation function
+## simfunc <- function(pars) {	
+## 	x <- rnorm(10,mean=pars["mu"],sd=pars["sigma"])    
+## 	c("T1"=median(x),"T2"=mad(x))	
+## }
 
 
 ###################################################
-### code chunk number 23: qle_with_R.Rnw:1486-1487
+### code chunk number 30: qle_with_R.Rnw:1525-1527 (eval = FALSE)
 ###################################################
-obs <- structure(c("T1"=2,"T2"=1),class="simQL")
+## lb <- c("mu"=0.5,"sigma"=0.1)
+## ub <- c("mu"=8.0,"sigma"=5.0)
 
 
 ###################################################
-### code chunk number 24: qle_with_R.Rnw:1490-1491
+### code chunk number 31: qle_with_R.Rnw:1533-1537 (eval = FALSE)
 ###################################################
-qsd <- getQLmodel(sim,lb,ub,obs,var.type="wcholMean")
+## sim <- simQLdata(sim=simfunc,
+##    nsim=10,N=8,lb=lb,ub=ub,method="maximinLHS")
+## # reset number of simulations (10 x 10)  
+## attr(sim,"nsim") <- 100
 
 
 ###################################################
-### code chunk number 25: qle_with_R.Rnw:1495-1497
+### code chunk number 32: qle_with_R.Rnw:1543-1544 (eval = FALSE)
+###################################################
+## obs <- structure(c("T1"=2,"T2"=1),class="simQL")
+
+
+###################################################
+### code chunk number 33: qle_with_R.Rnw:1547-1548 (eval = FALSE)
+###################################################
+## qsd <- getQLmodel(sim,lb,ub,obs,var.type="wcholMean")
+
+
+###################################################
+### code chunk number 34: qle_with_R.Rnw:1552-1553 (eval = FALSE)
+###################################################
+## QS <- qscoring(qsd, x0=c("mu"=5,"sigma"=3.0))
+
+
+###################################################
+### code chunk number 35: qle_with_R.Rnw:1555-1556
 ###################################################
-QS <- qscoring(qsd, x0=c("mu"=5,"sigma"=3.0))
 print(QS)
 
 
 ###################################################
-### code chunk number 26: qle_with_R.Rnw:1506-1514
+### code chunk number 36: qle_with_R.Rnw:1565-1571 (eval = FALSE)
 ###################################################
-OPT <- qle(qsd,
-        simfunc,		
-        nsim=10,
-        global.opts=list("maxeval"=50),
-        local.opts=list("lam_max"=1e-4,"weights"=0.5,
-        "useWeights"=FALSE,"test"=TRUE),
-		iseed=123)
-print(OPT)
+## OPT <- qle(qsd,
+##         simfunc,		
+##         nsim=20,
+##         global.opts=list("maxeval"=50),
+##         local.opts=list("lam_max"=1e-3,"weights"=0.5,
+##         "useWeights"=FALSE,"test"=TRUE),cl=cl)
 
 
 ###################################################
-### code chunk number 27: qle_with_R.Rnw:1518-1560
+### code chunk number 37: qle_with_R.Rnw:1573-1574
+###################################################
+OPT
+
+
+###################################################
+### code chunk number 38: qle_with_R.Rnw:1578-1620
 ###################################################
 op <- par(mfrow=c(1, 2), mar=c(5.1, 4.1, 1.1, 1.1),
 		oma=c(5,4,1,1),xaxs='i', yaxs='i',
@@ -327,13 +397,13 @@ par(op)
 
 
 ###################################################
-### code chunk number 28: qle_with_R.Rnw:1569-1570
+### code chunk number 39: qle_with_R.Rnw:1629-1630
 ###################################################
 checkMultRoot(OPT,verbose=TRUE)
 
 
 ###################################################
-### code chunk number 29: qle_with_R.Rnw:1578-1581
+### code chunk number 40: qle_with_R.Rnw:1638-1641
 ###################################################
 obs0 <- simQLdata(simfunc,X=OPT$par,nsim=1000,mode="matrix")[[1]]
 var(obs0)
@@ -341,15 +411,13 @@ attr(OPT$final,"Sigma")
 
 
 ###################################################
-### code chunk number 30: qle_with_R.Rnw:1584-1587
+### code chunk number 41: qle_with_R.Rnw:1643-1644 (eval = FALSE)
 ###################################################
-Stest <- qleTest(OPT,method=c("qscoring","bobyqa"),
-		   sim=simfunc,nsim=1000)
-Stest
+## stopCluster(cl)
 
 
 ###################################################
-### code chunk number 31: qle_with_R.Rnw:1611-1617
+### code chunk number 42: qle_with_R.Rnw:1674-1680
 ###################################################
 data(matclust)
 OPT <- matclust$OPT
@@ -360,27 +428,27 @@ library(spatstat)
 
 
 ###################################################
-### code chunk number 32: qle_with_R.Rnw:1621-1623
+### code chunk number 43: qle_with_R.Rnw:1684-1686
 ###################################################
 data(redwood)
 fitMat <- kppm(redwood, ~1, "MatClust")
 
 
 ###################################################
-### code chunk number 33: qle_with_R.Rnw:1626-1627
+### code chunk number 44: qle_with_R.Rnw:1689-1690
 ###################################################
 fitMat$modelpar
 
 
 ###################################################
-### code chunk number 34: qle_with_R.Rnw:1629-1631 (eval = FALSE)
+### code chunk number 45: qle_with_R.Rnw:1692-1694 (eval = FALSE)
 ###################################################
 ## RNGkind("L'Ecuyer-CMRG")
 ## set.seed(297)
 
 
 ###################################################
-### code chunk number 35: qle_with_R.Rnw:1635-1644
+### code chunk number 46: qle_with_R.Rnw:1698-1707
 ###################################################
 simStat <- function(X,cond){
  x <- Kest(X,r=cond$rr,correction="best")
@@ -394,7 +462,7 @@ simStat <- function(X,cond){
 
 
 ###################################################
-### code chunk number 36: qle_with_R.Rnw:1648-1652
+### code chunk number 47: qle_with_R.Rnw:1711-1715
 ###################################################
 simClust <- function(theta,cond){
  X <- rMatClust(theta["kappa"],theta["R"],theta["mu"],win=cond$win)	
@@ -403,7 +471,7 @@ simClust <- function(theta,cond){
 
 
 ###################################################
-### code chunk number 37: qle_with_R.Rnw:1657-1661
+### code chunk number 48: qle_with_R.Rnw:1720-1724
 ###################################################
 nsim <- 50
 Nsample <- 12
@@ -412,73 +480,91 @@ cond <- list(win=owin(c(0, 2),c(0, 2)),
 
 
 ###################################################
-### code chunk number 38: qle_with_R.Rnw:1664-1666
+### code chunk number 49: qle_with_R.Rnw:1727-1729
 ###################################################
 lb <- c("kappa"=20,"R"=0.01,"mu"=1)
 ub <- c("kappa"=30,"R"=0.25,"mu"=5)	 
 
 
 ###################################################
-### code chunk number 39: qle_with_R.Rnw:1670-1674 (eval = FALSE)
+### code chunk number 50: qle_with_R.Rnw:1733-1737 (eval = FALSE)
 ###################################################
-## cl <- makeCluster(8)
+## cl <- makeCluster(8L)
 ## clusterSetRNGStream(cl)
 ## clusterCall(cl,fun=function(x) library("spatstat", character.only=TRUE))
 ## clusterExport(cl=cl,varlist=c("simStat"), envir=environment())
 
 
 ###################################################
-### code chunk number 40: qle_with_R.Rnw:1678-1679 (eval = FALSE)
+### code chunk number 51: qle_with_R.Rnw:1741-1742 (eval = FALSE)
 ###################################################
 ## obs0 <- simStat(redwood,cond)
 
 
 ###################################################
-### code chunk number 41: qle_with_R.Rnw:1682-1684 (eval = FALSE)
+### code chunk number 52: qle_with_R.Rnw:1745-1747 (eval = FALSE)
 ###################################################
 ## sim <- simQLdata(sim=simClust,cond=cond,nsim=nsim,
 ##           method="randomLHS",lb=lb,ub=ub,N=Nsample,cl=cl)
 
 
 ###################################################
-### code chunk number 42: qle_with_R.Rnw:1687-1689 (eval = FALSE)
+### code chunk number 53: qle_with_R.Rnw:1750-1752 (eval = FALSE)
 ###################################################
 ## qsd <- getQLmodel(sim,lb,ub,obs0,criterion="qle",
 ##          var.type="kriging",verbose=TRUE)
 
 
 ###################################################
-### code chunk number 43: qle_with_R.Rnw:1699-1700 (eval = FALSE)
+### code chunk number 54: qle_with_R.Rnw:1762-1763 (eval = FALSE)
 ###################################################
 ## cvm <- prefitCV(qsd, reduce=FALSE, verbose=TRUE)
 
 
 ###################################################
-### code chunk number 44: qle_with_R.Rnw:1706-1707
+### code chunk number 55: qle_with_R.Rnw:1769-1770 (eval = FALSE)
 ###################################################
-crossValTx(qsd, cvm, type = "acve")
+## crossValTx(qsd, cvm, type = "acve")
 
 
 ###################################################
-### code chunk number 45: qle_with_R.Rnw:1712-1713
+### code chunk number 56: qle_with_R.Rnw:1772-1773
 ###################################################
-crossValTx(qsd, cvm, type = "mse")
+matclust$ACVE
 
 
 ###################################################
-### code chunk number 46: qle_with_R.Rnw:1724-1725
+### code chunk number 57: qle_with_R.Rnw:1778-1779 (eval = FALSE)
 ###################################################
-crossValTx(qsd, cvm, type = "ascve")
+## crossValTx(qsd, cvm, type = "mse")
 
 
 ###################################################
-### code chunk number 47: qle_with_R.Rnw:1735-1736
+### code chunk number 58: qle_with_R.Rnw:1781-1782
+###################################################
+matclust$MSE
+
+
+###################################################
+### code chunk number 59: qle_with_R.Rnw:1793-1794 (eval = FALSE)
+###################################################
+## crossValTx(qsd, cvm, type = "ascve")
+
+
+###################################################
+### code chunk number 60: qle_with_R.Rnw:1796-1797
+###################################################
+matclust$ASCVE
+
+
+###################################################
+### code chunk number 61: qle_with_R.Rnw:1807-1808
 ###################################################
 attr(cvm,"type") <- "max"
 
 
 ###################################################
-### code chunk number 48: qle_with_R.Rnw:1740-1743
+### code chunk number 62: qle_with_R.Rnw:1812-1815
 ###################################################
 x0 <- c("kappa"=24,"R"=0.08,"mu"=2.5)
 searchMinimizer(x0,qsd,info=TRUE,
@@ -486,7 +572,7 @@ searchMinimizer(x0,qsd,info=TRUE,
 
 
 ###################################################
-### code chunk number 49: qle_with_R.Rnw:1746-1749
+### code chunk number 63: qle_with_R.Rnw:1818-1821
 ###################################################
 qscoring(qsd,x0,
   opts=list("ftol_rel"=1e-6,"slope_tol"=1e-4),
@@ -494,7 +580,7 @@ qscoring(qsd,x0,
 
 
 ###################################################
-### code chunk number 50: qle_with_R.Rnw:1765-1772 (eval = FALSE)
+### code chunk number 64: qle_with_R.Rnw:1837-1844 (eval = FALSE)
 ###################################################
 ## qs.opts <-
 ##  list("xscale"=c(10,0.1,1),
@@ -506,7 +592,7 @@ qscoring(qsd,x0,
 
 
 ###################################################
-### code chunk number 51: qle_with_R.Rnw:1775-1791 (eval = FALSE)
+### code chunk number 65: qle_with_R.Rnw:1847-1863 (eval = FALSE)
 ###################################################
 ## OPT <- qle(qsd, simClust, cond=cond,  
 ##  qscore.opts = qs.opts,
@@ -520,39 +606,39 @@ qscoring(qsd,x0,
 ##                 "ftol_abs"=1e-2, # lower bound on criterion value, triggers testing
 ##                 "weights"=c(0.55), # constant weight factor
 ##                 "eta"=c(0.025,0.075), # ignored, automatic adjustment of weights
-##                 "test"=TRUE), # testing is enabled
+##                 "test"=TRUE), # testing approximate root is enabled
 ## 		method = c("qscoring","bobyqa","direct"), # restart methods		
 ## 		errType="max", # use max of kriging and CV error
-##         iseed=297, cl=cl) # set seed and pass cluster object for re-use
+##         iseed=297, cl=cl) # store seed and use given cluster object
 
 
 ###################################################
-### code chunk number 52: qle_with_R.Rnw:1794-1795
+### code chunk number 66: qle_with_R.Rnw:1866-1867
 ###################################################
 print(OPT)
 
 
 ###################################################
-### code chunk number 53: qle_with_R.Rnw:1798-1799
+### code chunk number 67: qle_with_R.Rnw:1870-1871
 ###################################################
 attr(OPT,"optInfo")
 
 
 ###################################################
-### code chunk number 54: qle_with_R.Rnw:1804-1805
+### code chunk number 68: qle_with_R.Rnw:1876-1877
 ###################################################
 OPT$final
 
 
 ###################################################
-### code chunk number 55: qle_with_R.Rnw:1809-1811
+### code chunk number 69: qle_with_R.Rnw:1881-1883
 ###################################################
 S0 <- searchMinimizer(OPT$par,OPT$qsd,
        method="bobyqa",cvm=OPT$cvm,verbose=TRUE)
 
 
 ###################################################
-### code chunk number 56: qle_with_R.Rnw:1816-1819
+### code chunk number 70: qle_with_R.Rnw:1888-1891
 ###################################################
 QS <- qscoring(OPT$qsd,OPT$par,
        opts=list("slope_tol"=1e-4,"score_tol"=1e-3),
@@ -560,20 +646,20 @@ QS <- qscoring(OPT$qsd,OPT$par,
 
 
 ###################################################
-### code chunk number 57: qle_with_R.Rnw:1823-1825
+### code chunk number 71: qle_with_R.Rnw:1895-1897
 ###################################################
 par <- rbind("QS"=QS$par,"S0"=S0$par)
 checkMultRoot(OPT,par=par)
 
 
 ###################################################
-### code chunk number 58: qle_with_R.Rnw:1828-1829
+### code chunk number 72: qle_with_R.Rnw:1900-1901
 ###################################################
-OPT$par
+QS$par
 
 
 ###################################################
-### code chunk number 59: qle_with_R.Rnw:1834-1845 (eval = FALSE)
+### code chunk number 73: qle_with_R.Rnw:1906-1917 (eval = FALSE)
 ###################################################
 ## par0 <- OPT$par 
 ## obs0 <- OPT$qsd$obs
@@ -584,36 +670,36 @@ OPT$par
 ##           obs0=obs0, # alternative observed statistics
 ##           sim=simClust,cond=cond,nsim=100,
 ##           method=c("qscoring","bobyqa","direct"), # restart methods
-##           opts=qs.opts, control=list("ftol_abs"=1e-8), # minimization options 
-##           multi.start=1L, cl=cl, verbose=TRUE) # multi-start and parallel options
+##           opts=qs.opts,control=list("ftol_abs"=1e-8), # minimization options 
+##           multi.start=1L,cl=cl,verbose=TRUE) # multi-start and parallel options
 
 
 ###################################################
-### code chunk number 60: qle_with_R.Rnw:1847-1848
+### code chunk number 74: qle_with_R.Rnw:1919-1920
 ###################################################
 print(Stest)
 
 
 ###################################################
-### code chunk number 61: qle_with_R.Rnw:1851-1852 (eval = FALSE)
+### code chunk number 75: qle_with_R.Rnw:1923-1924 (eval = FALSE)
 ###################################################
 ## stopCluster(cl)
 
 
 ###################################################
-### code chunk number 62: qle_with_R.Rnw:1861-1862
+### code chunk number 76: qle_with_R.Rnw:1933-1934
 ###################################################
 diag(attr(Stest,"qi"))^0.5
 
 
 ###################################################
-### code chunk number 63: qle_with_R.Rnw:1865-1866
+### code chunk number 77: qle_with_R.Rnw:1937-1938
 ###################################################
 sqrt(diag(attr(Stest,"msem")))
 
 
 ###################################################
-### code chunk number 64: qle_with_R.Rnw:1870-1871
+### code chunk number 78: qle_with_R.Rnw:1942-1943
 ###################################################
 attr(Stest,"msem")
 
