@@ -17,6 +17,35 @@ typedef enum { MEAN = 0, KRIG = 1} var_type;
 typedef enum { UNKNOWN = -1, DUAL = 0, VARIANCE = 1, BOTH = 2} krig_type;
 typedef enum { COPY_ZERO = 0, COPY_ONE = 1, COPY_MOD = 2, COPY_TRACE = 3} value_type;
 
+typedef struct krig_storage_s {
+	double *Qwork,
+	       *lambdak,
+		   *Rvec,
+		   *mu;
+
+	krig_storage_s() :
+		Qwork(0),lambdak(0),Rvec(0),mu(0)
+	{}
+
+	krig_storage_s(int lx, int fddim) :
+			Qwork(0),lambdak(0),Rvec(0),mu(0)
+	{
+		CALLOCX(mu,fddim, double);
+		CALLOCX(Rvec,fddim,double);
+		CALLOCX(lambdak,lx, double);
+		CALLOCX(Qwork,fddim*fddim,double);
+	}
+
+
+	~krig_storage_s(){
+		FREE(mu)
+		FREE(Rvec)
+		FREE(Qwork)
+		FREE(lambdak)
+	}
+
+} krig_storage_t, *krig_storage;
+
 typedef struct krig_result_s {
   double *mean,
          *sig2,
@@ -42,12 +71,12 @@ typedef struct krig_result_s {
 
   void alloc(int nCov, int lx) {
 	  if(nCov==0 || lx==0)
-	  	 error(_("number of covariance models or sample points must not be zero!"));
+	    ERR("number of covariance models or sample points must not be zero!");
 
 	  CALLOCX(w,nCov*lx,double);     // kriging weights as matrix: nCov rows, lx columns
 	  CALLOCX(mean,nCov,double);     // kriging means (at single point)
 	  if(krigType)
-	     CALLOCX(sig2,nCov,double);  // kriging variances (at single point) and nCov covariances
+	   CALLOCX(sig2,nCov,double);    // kriging variances (at single point) and nCov covariances
   }
 
 } krig_result_t, *krig_result;
@@ -62,7 +91,7 @@ typedef struct ql_options_s {
 
 	  SEXP R_varType = getListElement(R_opts, "varType");
 	  if(!isString(R_varType))
-	    error(_("Variance interpolation type should be a string: varType"));
+	    Rf_error(_("Variance interpolation type should be a string: varType"));
 	  const char *var_type = translateChar(asChar(R_varType));
 	  if( !strcmp("kriging",var_type) )  {
 	   varType = KRIG;
@@ -85,14 +114,17 @@ typedef struct krig_model_s {
          *Cmat,  /* Covariance matrix */
          *Cinv,  /* Inverse Covariance */
          *X1mat, /* X1 matrix, solving kriging equations */
-         *Qmat,  /* Q matri, solving kriging equations */
-         *dw;    /* Dual kriging weights */
+         *Qmat,  /* Q matrix, solving kriging equations */
+	     *dw;    /* Dual kriging weights */
+
+  double *f0,    /* trend vector*/
+         *s0; 	 /* and covariance vector work */
 
-  double *f0,*s0; /* trend and covariance vector work */
   int   trend,   /* trend order */
         fddim;   /* columns of trend matrix F*/
 
   cov_model cov;
+  krig_storage ks;
 
   krig_model_s(SEXP _R_cov, double *_Xmat, double *_data,
 		  int _lx, int _dx, krig_type _type,  value_type _cpy = COPY_ONE 	/* copy _data */ ) :
@@ -108,7 +140,8 @@ typedef struct krig_model_s {
 	      FREE(Cinv)
 	      FREE(X1mat)
 	      FREE(Qmat)
-	      FREE(dw)
+		  FREE(dw)
+		  DELETE(ks)
 	  } else {
 	      FREE(dw)
 	  }
@@ -116,17 +149,18 @@ typedef struct krig_model_s {
 	  FREE(f0)
 	  FREE(Fmat)
   	  FREE(Cmat)
-  	  if(cpy)
-  		FREE(data)
+
+	  if(cpy)
+  	   FREE(data)
   }
 
   void alloc();
 
   void setup(double *data);
 
-  void dualKriging(double *x, int nx, double *m);
+  void dualKriging(double *x, int nx, double *m, int &err);
 
-  void univarKriging(double *x, int nx, double *m, double *s, double *w);
+  void univarKriging(double *x, int nx, double *m, double *s, double *w, int &err);
 
 
 } krig_model_t, *krig_model;
@@ -134,35 +168,35 @@ typedef struct krig_model_s {
 
 typedef struct glkrig_models_s
 {
-  double *Xmat;
+  double *Xmat;	  /* design matrix */
 
   int lx,         /* rows of sample matrix */
       dx,         /* columns, dimension of sample points */
-      nCov;
+      nCov,
+	  info;
 
   value_type cpy;
   krig_type krigType;
-  int info;
 
   krig_model *km;
-  krig_result krigr[2];   /* working storage kriging: 0: default, 1: temporary */
+  krig_result krigr[2]; 		  /* working storage kriging: 0: default, 1: temporary */
 
-  glkrig_models_s(SEXP _R_covList, SEXP _R_Xmat, SEXP _R_data, SEXP R_krigType, int idx = 0, value_type _cpy = COPY_ONE) :
-	  Xmat(0), lx(0), dx(0), nCov(0), cpy(_cpy), info(0)
+  glkrig_models_s(SEXP _R_covList, SEXP _R_Xmat, SEXP _R_data, SEXP R_krigType,
+		  	  	  	  int idx = 0, value_type _cpy = COPY_ONE) :
+	  Xmat(0), lx(0), dx(0), nCov(0), info(0), cpy(_cpy)
   {
 	  int *dim = GET_DIMS(_R_Xmat);
-	  lx = dim[0];
-	  dx = dim[1];
+	  lx = dim[0]; dx = dim[1];
 
 	  if(isNull(_R_covList))
-		ERR("Covariance model is not set (NULL).");
+		ERR("Covariance model is NULL.");
 	  nCov = LENGTH(_R_covList);
 
-	  if(isNull(R_krigType))
+	  if(isNull(R_krigType)) {
 		 krigType = VARIANCE;
-	  else {
+	  } else {
 		  if(!isString(R_krigType))
-	  		error(_("Kriging type estimation should be a string"), "R_krigType");
+	  		Rf_error(_("Kriging type estimation should be a string: `R_krigType`."));
 		  const char *krig_type = translateChar(asChar(R_krigType));
 		  if( !strcmp("dual",krig_type) )
 			  krigType = DUAL;
@@ -173,8 +207,7 @@ typedef struct glkrig_models_s
 	  }
 
 	  /* kriging results storage */
-	  for(int i=0; i<2; i++)
-	  {
+	  for(int i=0; i<2; i++) {
 		if( (krigr[i] = new krig_result_s(nCov,lx,krigType)) == NULL)
 		  MEM_ERR(1,krig_result_s)
 	  }
@@ -189,8 +222,7 @@ typedef struct glkrig_models_s
 	  if( (km =  new(std::nothrow) krig_model[nCov]) == NULL)
 		  MEM_ERR(1,krig_model)
 
-	  for(int i=0; i<nCov; i++)
-	  {
+	  for(int i=0; i<nCov; i++) {
 		 if( ( km[i] = new(std::nothrow) krig_model_s(VECTOR_ELT(_R_covList,i),
 				 	 	 	 	  Xmat, REAL(AS_NUMERIC(VECTOR_ELT(AS_LIST(_R_data),i+idx))),
 								  lx,dx,krigType,cpy) ) == NULL )
@@ -201,12 +233,11 @@ typedef struct glkrig_models_s
 
   glkrig_models_s(SEXP _R_covList, double *_Xmat, double **_data, int _lx, int _dx,
 		  	  	  	 krig_type _krigType, int idx = 0, value_type _cpy = COPY_ONE) :
-		 Xmat(0), lx(_lx), dx(_dx), nCov(LENGTH(_R_covList)), cpy(_cpy), krigType(_krigType), info(0)
+		 Xmat(0), lx(_lx), dx(_dx), nCov(LENGTH(_R_covList)), info(0), cpy(_cpy), krigType(_krigType)
   {
 
   	  /* kriging results storage */
-  	  for(int i=0; i<2; i++)
-  	  {
+  	  for(int i=0; i<2; i++) {
   		if( (krigr[i] = new krig_result_s(nCov,lx,krigType)) == NULL)
   	  	  MEM_ERR(1,krig_result_s)
   	  }
@@ -240,11 +271,19 @@ typedef struct glkrig_models_s
 	   FREE(Xmat)
    }
 
-  void kriging(double *x, double *m, double *s, double *w);
-  inline void intern_kriging(double *x) {  kriging(x, krigr[0]->mean, krigr[0]->sig2, krigr[0]->w);}
+  void kriging(double *x, double *m, double *s, double *w, int &err);
 
-  void jacobian(double *x, double *mean, double *jac);
-  inline void intern_jacobian(double *x, double *jac) { jacobian(x, krigr[0]->mean, jac);  }
+  inline int intern_kriging(double *x) {
+	  kriging(x, krigr[0]->mean, krigr[0]->sig2, krigr[0]->w, info);
+	  return info;
+   }
+
+  void jacobian(double *x, double *mean, double *jac, double *fdwork, int &err);
+
+  inline int intern_jacobian(double *x, double *jac, double *fdwork) {
+	  jacobian(x, krigr[0]->mean, jac, fdwork, info);
+	  return info;
+  }
 
 } glkrig_models, *GLKM;
 
@@ -281,22 +320,31 @@ typedef struct cv_model_s {
 
 	void set(SEXP R_Xmat, SEXP R_data, SEXP R_cm);
 
-	void cvError(double *x, krig_model *km, double *cv);
+	void cvError(double *x, krig_model *km, double *cv, int &info);
 
 } cv_model_t, *CVKM;
 
-/*! \todo:  kriging models for var(T(X)) */
+typedef struct {
+	double *qimat,
+		   *vmat,
+		   *varS,
+		   *score;
+
+} ql_storage_t;
+
+
 typedef struct ql_data_s {
 
   // always to be allocated
   double *obs,          /* Statistics of reference parameter */
-         *qtheta,       /* work: nCov */
-		 *vmat,
+         *qtheta,       /* working array of length nCov, (T(x)-E[T(X)]) */
+		 *vmat,         /* variance matrix of statistics */
 		 *vmat_work,    /* variance matrix of statistics */
-		 *score_work,	/* working score vector */
-         *work,	        /* copy of diagonal terms of variance matrix of statistics */
+		 *work,	        /* copy of diagonal terms of variance matrix of statistics */
 		 *workx,		/* for kriging the variance matrix */
 		 *jactmp, 		/* transpose of jac or simply temporary storage */
+		 *fdwork,		/* working array for FD approxmiation ETX/dtheta (length: number of statistics) */
+		 *fdscore,		/* working array for FD approxmiation   Q/dtheta (length: number of parameters) */
 		 *Atmp, 	    /* temporary working matrix */
   	  	 *tmp;
 
@@ -306,20 +354,21 @@ typedef struct ql_data_s {
 
   // alloc and init
   ql_data_s(SEXP R_obs, SEXP R_Vmat, SEXP R_qlopts, int _nCov, int *_dims) :
-		  obs(0), qtheta(0), vmat(0), vmat_work(0), score_work(0), work(0),
-		  workx(0), jactmp(0), Atmp(0), tmp(0), qlopts(R_qlopts),
+		  obs(0), qtheta(0), vmat(0), vmat_work(0), work(0),
+		  workx(0), jactmp(0), fdwork(0), fdscore(0), Atmp(0), tmp(0), qlopts(R_qlopts),
 		  dx(_dims[1]), lx(_dims[0]), nCov(_nCov)
   {
 	 int nCov2 = SQR(nCov);
 
 	 CALLOCX(qtheta,nCov,double);
+	 CALLOCX(fdwork,nCov,double);
+	 CALLOCX(fdscore,dx,double);
 	 CALLOCX(Atmp,dx*nCov,double);
 	 CALLOCX(obs,nCov,double);
 	 CALLOCX(tmp,nCov,double);
 	 CALLOCX(jactmp,dx*nCov,double);
 	 CALLOCX(vmat,nCov2,double);
 	 CALLOCX(vmat_work,nCov2,double);
-	 CALLOCX(score_work,dx,double);
 
 	 for(int i=0;i<dx*nCov;i++)
 		jactmp[i]=Atmp[i]=0;
@@ -329,22 +378,17 @@ typedef struct ql_data_s {
 		 tmp[i]=obs[i]=_obs[i];
 	 }
 
-	 if(!isNull(R_Vmat))
-	 {
-		if(!isMatrix(R_Vmat))
-		  error(_("'%s' Variance is not of type matrix"), "R_Vmat");
-
-		 if(qlopts.useSigma || qlopts.varType == MEAN) {
-		    double *_vmat = REAL(R_Vmat);
-		    MEMCPY(vmat,_vmat,nCov2);
-		    MEMCPY(vmat_work,_vmat,nCov2);
-
-		    if(qlopts.varType == MEAN) {
-		      CALLOCX(work,nCov,double);
-		      splitDiag(vmat_work,nCov,work);
-		    }
-		 }
+	 if(qlopts.useSigma || qlopts.varType == MEAN) {
+		if(isNull(R_Vmat) || !isMatrix(R_Vmat))
+		  ERR("Variance `R_Vmat` is either NULL or not a matrix.");
+		double *_vmat = REAL(R_Vmat);
+		MEMCPY(vmat,_vmat,nCov2);
+		MEMCPY(vmat_work,_vmat,nCov2);
 
+		if(qlopts.varType == MEAN) {
+		  CALLOCX(work,nCov,double);
+		  splitDiag(vmat_work,nCov,work);
+		}
 	 } else {
 		 CALLOCX(workx,nCov2,double);
 	 }
@@ -353,13 +397,14 @@ typedef struct ql_data_s {
 
   ~ql_data_s() {
 	  FREE(jactmp)
+	  FREE(fdwork)
+	  FREE(fdscore)
 	  FREE(tmp)
 	  FREE(obs)
 	  FREE(Atmp)
 	  FREE(qtheta)
 	  FREE(vmat)
 	  FREE(vmat_work)
-	  FREE(score_work)
 	  FREE(work)
 	  FREE(workx)
   };
@@ -375,6 +420,7 @@ typedef struct ql_model_s {
 	GLKM varkm;  			/* Kriging models for variance matrices */
 	CVKM cvmod;				/* CV models (Cross-Validation kriging models) */
 	ql_data qld;
+    ql_storage_t qlsolve;		  /* general storage for solving kriging equations */
 
 	double *qimat,  /* expected Quasi-Information */
 		   *score,  /* quasi score vector */
@@ -388,7 +434,7 @@ typedef struct ql_model_s {
 	// R_qsd, R_qlopts, R_Vmat, R_cm, (Rboolean) type
 	ql_model_s(SEXP R_qsd, SEXP R_qlopts, SEXP R_Xmat, SEXP R_Vmat, SEXP R_cm, value_type _cpy) :
 			glkm(NULL), varkm(NULL), cvmod(NULL), qld(NULL),
-			qimat(0), score(0), jac(0), qiobs(0), lower(0), upper(0), info(0)
+			qimat(0), score(0), jac(0), qiobs(0), lower(0), upper(0),info(0)
 	{
 		/* bounds */
 		lower = REAL(getListElement( R_qsd, "lower" ));
@@ -415,8 +461,6 @@ typedef struct ql_model_s {
 
 		 if(!qld->qlopts.useSigma && qld->qlopts.varType)
 		 {
-			 if(!isNull(R_Vmat))
-			   ERR("R_Vmat is not NULL but should be if using kriging approximation of variance.");
 			 if(isNull(R_covL))
 			   ERR("Covariance model for kriging variance matrix is not set (Null).");
 
@@ -436,6 +480,12 @@ typedef struct ql_model_s {
 		 CALLOCX(qiobs,dxdx,double);
 		 CALLOCX(qimat,dxdx,double);
 		 CALLOCX(jac,dx*nCov,double);
+
+		 /* ql storage for solving */
+		 CALLOCX(qlsolve.score,dx,double);
+		 CALLOCX(qlsolve.qimat,dxdx,double);
+		 CALLOCX(qlsolve.varS,dxdx,double);
+		 CALLOCX(qlsolve.vmat,nCov2,double);
 	}
 
 	~ql_model_s()
@@ -449,65 +499,77 @@ typedef struct ql_model_s {
 	  FREE(qimat)
 	  FREE(jac)
 	  FREE(score)
+
+	  /* storage */
+	  FREE(qlsolve.score)
+	  FREE(qlsolve.qimat)
+	  FREE(qlsolve.varS)
+	  FREE(qlsolve.vmat)
 	}
 
-	inline void intern_cvError(double *x) {
+	inline int intern_cvError(double *x) {
 		  if(glkm->krigType && qld->qlopts.useCV) {
-		  	 cvmod->cvError(x,glkm->km,glkm->krigr[0]->sig2);
-		  	 if(info)
-		  	   WRR("`NaN` detected in cross-validation errors.")
+		  	 cvmod->cvError(x,glkm->km,glkm->krigr[0]->sig2,info);
+		  	 if(info != NO_ERROR)
+		  	   LOG_ERROR(info," in `cvError`.")
 		  }
+		  return info;
     }
 
-	  /* variance score vector, only for krigType `var` !!! */
-	inline void intern_varScore(double *vars)
+	 /* variance score vector, only for krigType `var` !!! */
+	inline int intern_varScore(double *vars)
 	{
-   		 matmult_diag_sqrt(qld->Atmp,nCov,dx,glkm->krigr[0]->sig2);
-		 matmult_trans(qld->Atmp,nCov,dx,qld->Atmp,nCov,dx,vars,&info);
-		 if(info){
-		   WRR("`NaN` detected in matrix multiplication.")
-		 }
+   		 matmult_diag_sqrt(qld->Atmp,nCov,dx,glkm->krigr[0]->sig2,info);
+   		 if(info > 0)
+   			LOG_WARNING(info, "`NaN` detected in `matmult_diag_sqrt`.")
+		 matmult_trans(qld->Atmp,nCov,dx,qld->Atmp,nCov,dx,vars,info);
+		 if(info > 0)
+			LOG_WARNING(info," `NaN` detected in `matmult_trans`.")
 
 #if DEBUG
 		printMatrix("Atmp (1)",qld->Atmp,&nCov,&dx);
 		printVector("sig2",glkm->krigr[0]->sig2,&nCov);
 		printMatrix("vars",vars,&dx,&dx);
 #endif
+		return info;
 
     }
 
-	inline void intern_quasiObs(double *x, double *score, double *qiobs);
+	int intern_quasiObs(double *x, double *score, double *qiobs);
 
-	inline void intern_quasiScore(double *jac, double *score) {
-		 quasiScore(glkm->krigr[0]->mean,jac,qld->vmat,score);
+	inline int intern_quasiScore(double *jac, double *score) {
+		 quasiScore(glkm->krigr[0]->mean,jac,qld->vmat,score,info);
+		 return info;
 	}
 
 	double intern_qfTrace(double *x);
     double intern_qfVarStat(double *x);
+    double qfValue(double *score, double *varS);
 
-    inline double intern_qfScoreStat(double *x) {
-	  	return qfScoreStat(x,jac,score,qimat);
-	}
+    int intern_qfScore(double *x){ return qfScore(x,jac, score, qimat); }
 
-	void varMatrix(double *x, double *s, double *vmat);
+    inline double intern_qfValue() { return qfValue(score,qimat); }
+    inline double intern_qfScoreStat(double *x) { return qfScoreStat(x,jac,score,qimat); }
 
-	double qfValue(double *score, double *varS);
+	void varMatrix(double *x, double *s, double *vmat, int &err);
+
+	int qfScore(double *x, double *jac, double *score, double *qimat);
 	double qfScoreStat(double *x, double *jac, double *score, double *qimat);
 
-	void intern_quasiInfo(double *jac, double *qimat);
-	void quasiScore(double *mean, double *jac, double *vmat, double *score);
+	int intern_quasiInfo(double *jac, double *qimat);
+	void quasiScore(double *mean, double *jac, double *vmat, double *score, int &err);
 
-	double intern_mahalValue(double *x);			//use constant (inverted) variance
-	double intern_mahalVarTrace(double *x);			//use constant (inverted) variance
+	double intern_mahalValue(double *x);			// use constant (inverted) variance
+	double intern_mahalVarTrace(double *x);			// use constant (inverted) variance
 	double intern_mahalValue_theta(double *x);		// computing inverse variance
 
 
 } ql_model_t, *ql_model;
 
 //////////////////////////// wrapper ///////////////////////////////////////////////////////
-void addVar(double *sig2, int nc, double *vmat, double *work);
-void wrap_intern_kriging(double *x, void *data,  double *mean);
-void wrap_intern_quasiScore(double *x, void *data, double *score);
+int  addVar(double *sig2, int nc, double *vmat, double *work);
+void wrap_intern_kriging(double *x, void *data,  double *mean, int *err);
+void wrap_intern_quasiScore(double *x, void *data, double *score, int *err);
 void setVmatAttrib(ql_model qlm, SEXP R_VmatNames, SEXP R_ans);
 
 ///////////////////// KRIGING /////////////////////////////////////////////////////////////////////////////////////
